#include "tests_tile_segmentation.h"

template<typename Type, class create_func, class ref_func, class run_calc_ptr,
         class cmp_func>
bool test_min_val(Type* src, create_func create_vector, ref_func reference,
                  run_calc_ptr run_calc, cmp_func comparator, int size,
                  int* localmem) {
    create_vector(src, size, 0);

    FLUSH_ALL_CACHES();
    uint32_t tic_count[2], instruction_count[2];
    count_tics(tic_count, instruction_count);
    Type ref_res = reference(src, size);
    count_tics(&tic_count[1], &instruction_count[1]);

    std::cout<<"Ref func result (size = "<<size<<"): tic = "
             <<tic_count[1] - tic_count[0]<<" instr = "
             <<instruction_count[1] - instruction_count[0]<<std::endl;

    TileSegConfig config;
    CreateTileSegConfigMinVal(src, size, &config, localmem);
    FLUSH_ALL_CACHES();

    count_tics(tic_count, instruction_count);
    Type opt_res = run_calc(&config);
    count_tics(&tic_count[1], &instruction_count[1]);

    std::cout<<"Opt func result (size = "<<size<<"): tic = "
             <<tic_count[1] - tic_count[0]<<" instr = "
             <<instruction_count[1] - instruction_count[0]<<std::endl;

    int ret = comparator(&ref_res, &opt_res, 1);
    return ret;
}

int main()
{
    uint32_t l2dc_val = *L2DC;
    FLUSH_ALL_CACHES();
    *L2DC &= 0xFFFFFFBF;

    void* src = memalign(64, SIZE * sizeof(int64_t));

    int ret = 0;
    int test_status = 0;

    for (int i = 1; i <= SIZE; i*=2) {
        std::cout << "minval_s16" << std::endl;
        #ifdef USE_REF_VER
        ret = test_min_val(static_cast<int16_t*>(src), create_vector_s16,
                           ref_minval, RunCalculationMinVal16, compare_s16, i,
                           &__local_mem);
        #else
        ret = test_min_val(static_cast<int16_t*>(src), create_vector_s16,
                           minval, RunCalculationMinVal16, compare_s16, i,
                           &__local_mem);
        #endif

        test_status |= ret;
        if (ret)
            std::cout << "minval16 error!\n";

        std::cout << "minval_s32" << std::endl;
        #ifdef USE_REF_VER
        ret = test_min_val(static_cast<int32_t*>(src), create_vector_s32,
                           ref_minval32, RunCalculationMinVal32, compare_s32, i,
                           &__local_mem);
        #else
        ret = test_min_val(static_cast<int32_t*>(src), create_vector_s32,
                           minval32, RunCalculationMinVal32, compare_s32, i,
                           &__local_mem);
        #endif

        test_status |= ret;
        if (ret)
            std::cout << "minval32 error!\n";

        std::cout << "minval_fl" << std::endl;
        #ifdef USE_REF_VER
        ret = test_min_val(static_cast<float*>(src), create_vector_float,
                          ref_minval_fl, RunCalculationMinValFl, compare_float,
                          i, &__local_mem);
        #else
        ret = test_min_val(static_cast<float*>(src), create_vector_float,
                          minval_fl, RunCalculationMinValFl, compare_float,
                          i, &__local_mem);
        #endif

        test_status |= ret;
        if (ret)
            std::cout << "minval_fl error!\n";

        std::cout << "minval_db" << std::endl;
        #ifdef USE_REF_VER
        ret = test_min_val(static_cast<double*>(src), create_vector_double,
                           ref_minval_db, RunCalculationMinValDb,
                           compare_double, i, &__local_mem);
        #else
        ret = test_min_val(static_cast<double*>(src), create_vector_double,
                           minval_db, RunCalculationMinValDb,
                           compare_double, i, &__local_mem);
        #endif

        test_status |= ret;
        if (ret)
            std::cout << "minval_db error!\n";

        if (!test_status)
            std::cout << "Test passed" << std::endl;
        else
            std::cout << "Test failed" << std::endl;
    }

    free(src);

    *L2DC = l2dc_val;

    return test_status;
}
